package main;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.Component;
import java.util.List;
import java.io.File;

/**
 * classe PanneauListeEtudiants permettant d'afficher une liste d'étudiants
 * par une JList
 */
public class PanneauListeEtudiants extends JPanel {

	JList jListEtudiants;//ajouter

	public PanneauListeEtudiants(List<Etudiant> etudiants) {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

		// titre du panneau contenant la liste des étudiants
		JLabel titrePanneau = new JLabel("Liste des etudiants", JLabel.CENTER);
		titrePanneau.setBackground(Color.BLUE);
		titrePanneau.setOpaque(true);
		titrePanneau.setForeground(Color.WHITE);
		add(titrePanneau, BorderLayout.NORTH);
            
            //TO DO
		// liste des etudiants dans une JList. Par defaut appelle
		// la methode toString() de chaque etudiant (trop d'infos)
		//
		// Pour affichage seulement, la ligne suivante suffit 
		//		jListEtudiants = new JList(etudiants.toArray());
		// Pour pouvoir faire des modifications sur le modele :
		DefaultListModel modeleListeEtudiants = new DefaultListModel();
		for (Etudiant etudiant : etudiants) {
			modeleListeEtudiants.addElement(etudiant);
		}
		jListEtudiants = new JList(modeleListeEtudiants);

		// on associe un ListCellRenderer à la JList pour definir les
		// donnees d'un etudiant qui seront affiches
		jListEtudiants.setCellRenderer(new EtudiantCellRenderer());

		jListEtudiants.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		jListEtudiants.setVisibleRowCount(15);

		add(new JScrollPane(jListEtudiants), BorderLayout.EAST);
	}

	// méthodes pour gérer la liste d'étudiants affichés
	
	/**
 	 * obtenir l'etudiant selectionné
 	 */ 
	public Etudiant getSelectedEtudiant() {
		if (jListEtudiants.isSelectionEmpty()) {
			return null;
		}
		return (Etudiant) jListEtudiants.getSelectedValue();
	}

	/**
 	 * ajouter un etudiant au modele de la JList
 	 */ 
	public void ajouterEtudiant(Etudiant etudiant) {
		((DefaultListModel) jListEtudiants.getModel()).addElement(etudiant);
	}

	/**
 	 * remplacer l'etudiant selectionne dans la JList par un autre
 	 */ 
	public void modifierEtudiant(Etudiant etudiant) {
		((DefaultListModel) jListEtudiants.getModel()).set(
			jListEtudiants.getSelectedIndex(),
			etudiant);
	}

	public void supprimerEtudiantSelectionne() {
		((DefaultListModel) jListEtudiants.getModel()).remove(
			jListEtudiants.getSelectedIndex());
	}
}

/**
 * classe permettant de régler le rendu de chaque élément de la liste.
 * Un étudiant sera représenté par :
 * 	
 */
class EtudiantCellRenderer extends JLabel implements ListCellRenderer {
	// icoes pour associer à un etudiant
	private final static ImageIcon ICON_VERY_GOOD = 
		new ImageIcon("src"+File.separator+"images" + File.separator + "genie.gif");
	private final static ImageIcon ICON_GOOD = 
		new ImageIcon("src"+File.separator+"images" + File.separator + "good.gif");
	private final static ImageIcon ICON_BAD = 
		new ImageIcon("src"+File.separator+"images" + File.separator + "bad.gif");
	private final static ImageIcon ICON_VERY_BAD = 
		new ImageIcon("src"+File.separator+"images" + File.separator + "very_bad.gif");

	// couleurs d'affichage
	private final static Color COULEUR_TEXTE_SELECTIONNE = Color.RED;
	private final static Color COULEUR_TEXTE = Color.BLACK;
	private final static Color COULEUR_FOND_MATH = Color.LIGHT_GRAY;
	private final static Color COULEUR_FOND_INFO = Color.CYAN;

	/**
 	 * methode appelee pour dessiner chaque element de la liste
 	 */ 
	public Component getListCellRendererComponent(
		JList list,              // la JList affichant les etudiants
		Object value,            // etudiant à afficher
		int index,               // numéro de l'étudiant dans la liste
		boolean isSelected,      // l'élement est-il selectionné ?
		boolean cellHasFocus)    // l'élément a-t-il le focus ?
	{
		Etudiant etudiant = (Etudiant) value;
		
		// on affiche uniquement les attributs nom et prénom de l'étudiant
		String texte = etudiant.getNom() + " " + etudiant.getPrenom();
		// on ajoute le numéro de l'étudiant dans la liste
		texte = (index + 1) + " - " + texte + "  ";
		setText(texte);
		
		// on ajoute une icône suivant la moyenne de l'élève
		if (etudiant.getMoyenne() < 5.0) {
			setIcon(ICON_VERY_BAD);
		} else if (etudiant.getMoyenne() < 10.0) {
			setIcon(ICON_BAD);
		} else if (etudiant.getMoyenne() < 15.0) {
			setIcon(ICON_GOOD);
		} else {
			setIcon(ICON_VERY_GOOD);
		}
			
		// fixe la couleur du texte suivant que l'élément est selectionné
		// ou pas
		if (isSelected) {
			setForeground(COULEUR_TEXTE_SELECTIONNE);
		} else {
			setForeground(COULEUR_TEXTE);
		}

		// fixe la couleur du fond suivant la spécialité de l'étudiant
		if (etudiant.getSpecialite() == Specialite.Mathematique) {
			setBackground(COULEUR_FOND_MATH);
		} else {
			setBackground(COULEUR_FOND_INFO);
		}

		// on reprend les attributs de la JList : composant actif et fonte
		setEnabled(list.isEnabled());
		setFont(list.getFont());
		setOpaque(true); // pour utiliser une couleur de fond
		return this;
	}
}
